import pstats

from yappi import YFuncStats  # type: ignore[attr-defined]

def clear_stats() -> None: ...
def set_clock_type(clock_type: str) -> None: ...
def start() -> None: ...
def stop() -> None: ...
def get_func_stats() -> YFuncStats: ...
def convert2pstats(stats: YFuncStats) -> pstats.Stats: ...
