"""Extend group code size

Revision ID: c2f508437a7e
Revises: d09424392322
Create Date: 2025-12-22 23:26:24.089512

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "c2f508437a7e"
down_revision = "d09424392322"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "group",
        "code",
        existing_type=sa.VARCHAR(length=1),
        type_=sa.String(length=2),
        existing_nullable=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "group",
        "code",
        existing_type=sa.String(length=2),
        type_=sa.VARCHAR(length=1),
        existing_nullable=False,
    )
    # ### end Alembic commands ###
