"""Make some columns not nullable

Revision ID: 1719e9432d78
Revises: 6e28ca2ef0c7
Create Date: 2023-09-04 00:05:23.930630

"""

from alembic import op
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "1719e9432d78"
down_revision = "6e28ca2ef0c7"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("team", "flag_url", existing_type=mysql.VARCHAR(length=100), nullable=False)
    op.alter_column(
        "team",
        "internal_flag_url",
        existing_type=mysql.VARCHAR(length=300),
        nullable=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "team",
        "internal_flag_url",
        existing_type=mysql.VARCHAR(length=300),
        nullable=True,
    )
    op.alter_column("team", "flag_url", existing_type=mysql.VARCHAR(length=100), nullable=True)
    # ### end Alembic commands ###
