"""Add unique constraint to match reference

Revision ID: eb38661b5a0d
Revises: 1719e9432d78
Create Date: 2025-07-14 22:29:38.502892

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "eb38661b5a0d"
down_revision = "1719e9432d78"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "uq_columns_group_id_index",
        "match_reference",
        ["group_id", "index"],
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("uq_columns_group_id_index", "match_reference", type_="unique")
    # ### end Alembic commands ###
